<?php

namespace Luticate\Utils\Business;

use Luticate\Utils\Dbo\LuDbo;
use Luticate\Utils\LuDataAccess;

abstract class LuBusiness {

    /**
     * @param string $reason
     * @throws LuBusinessException
     */
    public static function unauthorized($reason = 'Unauthorized')
    {
        throw new LuBusinessException($reason, 401);
    }

    /**
     * @param string $reason
     * @throws LuBusinessException
     */
    public static function notFound($reason = 'Resource not found')
    {
        throw new LuBusinessException($reason, 404);
    }

    /**
     * @param string $reason
     * @throws LuBusinessException
     */
    public static function badInput($reason = 'Invalid user input')
    {
        throw new LuBusinessException($reason, 400);
    }

    /**
     * @param $id int
     * @return LuDbo|null
     */
    public static function getById($id)
    {
        $data = static::getDataAccess()->getById($id);
        if (is_null($data))
            self::notFound(static::getResourceName() . " not found");
        return $data;
    }

    /**
     * @param $id int
     * @return true
     */
    public static function deleteById($id)
    {
        $res = static::getDataAccess()->deleteById($id);
        if (!$res)
            self::notFound(static::getResourceName() . " not found");
        return true;
    }

    /**
     * @return LuDataAccess
     */
    protected static function getDataAccess()
    {
        return null;
    }

    public static function getAll($page = 0, $perPage = PHP_INT_MAX, $query = "")
    {
        return static::getDataAccess()->getAll($page, $perPage, $query);
    }

    public static function getResourceName()
    {
        $match = [];
        preg_match('/([^\\\\]*)Business$/', get_called_class(), $match);
        return $match[1];
    }
}
