<?php
use Luticate\Utils\Dbo\LuDboDeserializeException;
use Luticate\Utils\Dbo\LuIntDboArray;
use Luticate\Utils\Dbo\LuIntDbo;

/**
 * Created by PhpStorm.
 * User: robin
 * Date: 5/29/16
 * Time: 2:57 PM
 */

class LuIntDboArrayTest extends \PHPUnit_Framework_TestCase
{

    public function testArrayEmpty()
    {
        $dbo = LuIntDboArray::jsonDeserialize([]);
        $this->assertSame([], $dbo->getArray());
    }
    
    public function testArrayString()
    {
        $dbo = LuIntDboArray::jsonDeserialize(["42"]);
        $this->assertSame([42], $dbo->getArray());
    }

    public function testArrayMultipleString()
    {
        $dbo = LuIntDboArray::jsonDeserialize(["42", "24", "48"]);
        $this->assertSame([42, 24, 48], $dbo->getArray());
    }

    public function testArrayInt()
    {
        $dbo = LuIntDboArray::jsonDeserialize([42]);
        $this->assertSame([42], $dbo->getArray());
    }

    public function testArrayMultipleInt()
    {
        $dbo = LuIntDboArray::jsonDeserialize([42, 24, 48]);
        $this->assertSame([42, 24, 48], $dbo->getArray());
    }

    public function testArrayStringIntPlusData()
    {
        $this->expectException(LuDboDeserializeException::class);
        LuIntDboArray::jsonDeserialize(["42test"]);
    }

    public function testArrayMultipleStringIntPlusData()
    {
        $this->expectException(LuDboDeserializeException::class);
        LuIntDboArray::jsonDeserialize(["42", 42, "42test"]);
    }
    
    public function testIntString()
    {
        $this->expectException(LuDboDeserializeException::class);
        LuIntDboArray::jsonDeserialize("42");
    }
    
    public function testInt()
    {
        $this->expectException(LuDboDeserializeException::class);
        LuIntDboArray::jsonDeserialize(42);
    }
    public function testIntNegativeString()
    {
        $this->expectException(LuDboDeserializeException::class);
        LuIntDboArray::jsonDeserialize("-42");
    }

    public function testIntNegative()
    {
        $this->expectException(LuDboDeserializeException::class);
        LuIntDboArray::jsonDeserialize(-42);
    }

    public function testFloat()
    {
        $this->expectException(LuDboDeserializeException::class);
        LuIntDboArray::jsonDeserialize(42.42);
    }

    public function testFloatString()
    {
        $this->expectException(LuDboDeserializeException::class);
        LuIntDboArray::jsonDeserialize("42.42");
    }

    public function testExponentialString()
    {
        $this->expectException(LuDboDeserializeException::class);
        LuIntDboArray::jsonDeserialize("42e+01");
    }

    public function testBool()
    {
        $this->expectException(LuDboDeserializeException::class);
        LuIntDboArray::jsonDeserialize(true);
    }

    public function testBoolString()
    {
        $this->expectException(LuDboDeserializeException::class);
        LuIntDboArray::jsonDeserialize("true");
    }

    public function testArrayFloat()
    {
        $this->expectException(LuDboDeserializeException::class);
        LuIntDboArray::jsonDeserialize([42.42]);
    }

    public function testNull()
    {
        $this->expectException(LuDboDeserializeException::class);
        LuIntDboArray::jsonDeserialize(null);
    }

    public function testObject()
    {
        $this->expectException(LuDboDeserializeException::class);
        LuIntDboArray::jsonDeserialize(new self());
    }
}